@@ -38,8 +38,9 @@ class NodeBase(object):
         """
         args = kwargs
         args.update({'parent' : self, 'child' : descendant })
+        disable_check = args.pop('disable_circular_check', False)
         cls = self.children.through(**kwargs)
-        return cls.save()
+        return cls.save(disable_circular_check=disable_check)
 
 
     def add_parent(self, parent, *args, **kwargs):
@@ -135,19 +136,19 @@ class NodeBase(object):
         """
         Check if has children and not ancestors
         """
-        return bool(self.children.count() and not self.ancestors_set())
+        return bool(self.children.exists() and not self._parents.exists())
 
     def is_leaf(self):
         """
         Check if has ancestors and not children
         """
-        return bool(self.ancestors_set() and not self.children.count())
+        return bool(self._parents.exists() and not self.children.exists())
 
     def is_island(self):
         """
         Check if has no ancestors nor children
         """
-        return bool(not self.is_root() and not self.is_leaf())
+        return bool(not self.children.exists() and not self._parents.exists())
 
     def _get_roots(self, at):
         """
@@ -217,7 +218,7 @@ def edge_factory(node_model, child_to_field = "id", parent_to_field = "id", conc
         except IndexError:
             node_model_name = node_model
     else:
-        node_model_name = node_mode._meta.module_name
+        node_model_name = node_model._meta.module_name
 
     class Edge(base_model):
         class Meta:
@@ -230,7 +231,8 @@ def edge_factory(node_model, child_to_field = "id", parent_to_field = "id", conc
             return "%s is child of %s" % (self.child, self.parent)
 
         def save(self, *args, **kwargs):
-            self.parent.__class__.circular_checker(self.parent, self.child)
+            if not kwargs.pop('disable_circular_check', False):
+                self.parent.__class__.circular_checker(self.parent, self.child)
             super(Edge, self).save(*args, **kwargs) # Call the "real" save() method.
 
     return Edge
@@ -248,7 +250,8 @@ def node_factory(edge_model, children_null = True, base_model = models.Model):
                 null        = children_null,
                 blank       = children_null,
                 symmetrical = False,
-                through     = edge_model)
+                through     = edge_model,
+                related_name = '_parents') # NodeBase.parents() is a function
 
     return Node
 
